@@ -1,6 +1,6 @@
 import re
 import uuid
-from typing import Optional
+from typing import Any, Dict, Optional
 
 from boto3 import Session
 from botocore.client import BaseClient
@@ -29,8 +29,8 @@ from sagemaker_studio.models.execution import (
 from sagemaker_studio.projects import ProjectService
 from sagemaker_studio.utils._internal import InternalUtils
 
-DEFAULT_INSTANCE_TYPE = "ml.m4.xlarge"  # consistent with the default instance type when create a training job via SageMaker console
-DEFAULT_IMAGE_VERSION = "2.2"  # consistent with default image version in Space
+DEFAULT_INSTANCE_TYPE = "ml.m6i.xlarge"  # consistent with the default instance in the toolkit
+DEFAULT_IMAGE_VERSION = "2.8"  # consistent with default image version in Space
 
 
 class RemoteExecutionClient(ExecutionClient):
@@ -60,24 +60,23 @@ class RemoteExecutionClient(ExecutionClient):
         self.datazone_api: BaseClient = datazone_api
         self.project_api: ProjectService = project_api
         self.config: ExecutionConfig = config
-        if not self.config.domain_identifier or not self.config.project_identifier:
-            raise InternalServerError("Domain identifier and project identifier are required")
-        self.domain_identifier = self.config.domain_identifier
-        self.project_identifier = self.config.project_identifier
-        self.datazone_stage = self.config.datazone_stage
-        self.datazone_endpoint = self.config.datazone_endpoint
-        self.datazone_domain_region = self.config.datazone_domain_region
-        self.project_s3_path = self.config.project_s3_path
-        if not self.config.datazone_environment_id:
-            raise InternalServerError("DataZone environment id is required")
-        self.datazone_environment_id = self.config.datazone_environment_id
         self.session = Session()
-        self.__set_default_tooling_environment()
-        self.__set_sagemaker_environment()
-        self.__set_sagemaker_client()
-        self.__set_ec2client()
-        self.__set_ssmclient()
-        self.__set_stack()
+        self.domain_identifier = None
+        self.project_identifier = None
+        self.datazone_stage = None
+        self.datazone_endpoint = None
+        self.datazone_domain_region = None
+        self.project_s3_path = None
+        self.datazone_environment_id = None
+        self.default_tooling_environment = None
+        self.sagemaker_environment = None
+        self.sagemaker_client = None
+        self.ec2_client = None
+        self.ssm_client = None
+        self.security_group = None
+        self.subnets = None
+        self.user_role_arn = None
+        self.kms_key_identifier = None
 
     """
     Retrieves detailed information about a specific execution.
@@ -86,9 +85,7 @@ class RemoteExecutionClient(ExecutionClient):
     execution identifier.
 
     Args:
-        request (GetExecutionRequest): An object containing the necessary information to identify
-        the execution. This typically includes:
-        - execution_id: The unique identifier of the execution to retrieve
+        execution_id (str): The unique identifier of the execution to retrieve
 
     Returns:
         dict: A dictionary containing detailed information about the execution. The structure
@@ -126,7 +123,13 @@ class RemoteExecutionClient(ExecutionClient):
           applicable to the current state of the execution.
     """
 
-    def get_execution(self, request: GetExecutionRequest) -> dict:
+    def get_execution(self, execution_id: str) -> dict:
+        parameters = {"execution_id": execution_id}
+        request = GetExecutionRequest(**parameters)
+        return self._get_execution_internal(request)
+
+    def _get_execution_internal(self, request: GetExecutionRequest) -> dict:
+        self.__setup_execution_client()
         self.__validate_default_environment(self.default_tooling_environment)
 
         training_job_arn = "arn:aws:sagemaker:{0}:{1}:training-job/{2}".format(
@@ -175,6 +178,13 @@ class RemoteExecutionClient(ExecutionClient):
                     "S3ModelArtifacts", None
                 ),
                 "tags": execution_tags,
+                "metricDefinitions": (
+                    describe_training_job_response.get("AlgorithmSpecification", {}).get(
+                        "MetricDefinitions", []
+                    )
+                    if "AlgorithmSpecification" in describe_training_job_response
+                    else []
+                ),
             }
 
             if "FailureReason" in describe_training_job_response:
@@ -209,16 +219,14 @@ class RemoteExecutionClient(ExecutionClient):
     using the criteria specified in the ListExecutionsRequest object.
 
     Args:
-        request (ListExecutionsRequest): An object containing the parameters for listing executions.
-            This may include:
-            - start_time_after: Filter executions starting after a specific time
-            - name_contains: Filter executions whose name contains a specific string
-            - status: Filter executions by their current status (e.g., 'IN_PROGRESS', 'COMPLETED', 'FAILED', 'STOPPING', 'STOPPED')
-            - filter_by_tags: Filter executions containing specific tags
-            - max_results: Maximum number of results to return
-            - next_token: Token for pagination
-            - sort_by: Field to sort the results by (e.g., 'start_time', 'status', 'end_time', 'name')
-            - sort_order: ASCENDING or DESCENDING order
+        start_time_after (Optional[int]): Filter executions starting after a specific time
+        name_contains (Optional[str]): Filter executions whose name contains a specific string
+        status (Optional[str]): Filter executions by their current status (e.g., 'IN_PROGRESS', 'COMPLETED', 'FAILED', 'STOPPING', 'STOPPED')
+        sort_order (Optional[str]): Order to return results (e.g., 'ASCENDING' or 'DESCENDING')
+        sort_by (Optional[str]): Field to sort the results by (e.g., 'NAME', 'STATUS', 'START_TIME', 'END_TIME')
+        next_token (Optional[str]): Token for pagination
+        max_results (Optional[int]): Maximum number of results to return
+        filter_by_tags (Optional[dict]): Filter executions containing specific tags
 
     Returns:
         dict: A dictionary containing the list of executions and metadata. Typical structure:
@@ -254,7 +262,39 @@ class RemoteExecutionClient(ExecutionClient):
             print(f"Execution ID: {execution['id']}, Status: {execution['status']}")
     """
 
-    def list_executions(self, request: ListExecutionsRequest) -> dict:
+    def list_executions(
+        self,
+        start_time_after: Optional[int] = None,
+        name_contains: Optional[str] = None,
+        status: Optional[str] = None,
+        sort_order: Optional[str] = None,
+        sort_by: Optional[str] = None,
+        next_token: Optional[str] = None,
+        max_results: Optional[int] = None,
+        filter_by_tags: Optional[dict] = None,
+    ) -> dict:
+        parameters = {}
+        if start_time_after is not None:
+            parameters["start_time_after"] = start_time_after
+        if name_contains is not None:
+            parameters["name_contains"] = name_contains
+        if status is not None:
+            parameters["status"] = status
+        if sort_order is not None:
+            parameters["sort_order"] = sort_order
+        if sort_by is not None:
+            parameters["sort_by"] = sort_by
+        if next_token is not None:
+            parameters["next_token"] = next_token
+        if max_results is not None:
+            parameters["max_results"] = max_results
+        if filter_by_tags is not None:
+            parameters["filter_by_tags"] = filter_by_tags
+        request = ListExecutionsRequest(**parameters)
+        return self._list_executions_internal(request)
+
+    def _list_executions_internal(self, request: ListExecutionsRequest) -> dict:
+        self.__setup_execution_client()
         try:
             search_args = {"Resource": "TrainingJob"}
             search_status = None
@@ -278,6 +318,11 @@ class RemoteExecutionClient(ExecutionClient):
                 request.get("filter_by_tags"),
             )
             search_args["SearchExpression"] = search_expression
+            # A recent change to project role now requires the SearchVisibilityCondition when using
+            # the SageMaker Search API to enforce project-level access controls.
+            search_args["VisibilityConditions"] = [
+                {"Key": "Tags.AmazonDataZoneProject", "Value": self.project_identifier}
+            ]
             if request.get("next_token"):
                 search_args["NextToken"] = request.next_token
             search_response = self.sagemaker_client.search(
@@ -342,14 +387,14 @@ class RemoteExecutionClient(ExecutionClient):
     the configuration specified in the StartExecutionRequest object.
 
     Args:
-        request (StartExecutionRequest): An object containing the necessary
-        parameters to start a new execution. This typically includes:
-        - execution_name: A unique name for the execution
-        - execution_type: The type of execution (e.g., 'NOTEBOOK')
-        - input_config: Configuration for the input data for the execution
-        - compute:  # Configuration for the compute environment
-        - output_config: Configuration for the outputs like output formats
-        - termination_condition: Condition for terminating the execution
+        execution_name (str): A unique name for the execution
+        execution_type (str): The type of execution (e.g., 'NOTEBOOK')
+        input_config (Dict[str, Any]): Configuration for the input data for the execution
+        compute (Optional[str]): Configuration for the compute environment
+        output_config (Optional[Dict[str, Any]]): Configuration for the outputs like output formats
+        client_token (Optional[str]): A unique token to ensure idempotency
+        termination_condition (Optional[Dict[str, Any]]): Condition for terminating the execution
+        tags (Optional[Dict[str, Any]]): Tags related to the execution
 
     Returns:
         dict: A dictionary containing information about the started execution. Typical structure:
@@ -374,21 +419,41 @@ class RemoteExecutionClient(ExecutionClient):
             input_config={"notebook_config": {"input_path": "src/folder2/test.ipynb"}},
             execution_type="NOTEBOOK",
             output_config={"notebook_config": {"output_formats": ["NOTEBOOK", "HTML"]}},
-            termination_condition={"max_runtime_in_seconds": 9000}
+            termination_condition={"max_runtime_in_seconds": 9000},
             compute={
                 "instance_type": "ml.c5.xlarge",
                 "image_details": {
                     # provide either ecr_uri or (image_name and image_version)
-                    "image_name": "sagemaker-distribution-embargoed-prod",
-                    "image_version": "2.2", // valid values - {2.2.0, 2.2, 2, 3.0.0, 3.0, 3}
-                    "ecr_uri": "123456123456.dkr.ecr.us-west-2.amazonaws.com/sagemaker-distribution-embargoed-prod:2.2-reinvent2024-cpu"
-                }
+                    "image_name": "sagemaker-distribution-prod",
+                    "image_version": "2.8",  // valid values - {2.6, 2.8, 2, 3.0, 3}
+                    "ecr_uri": "123456123456.dkr.ecr.us-west-2.amazonaws.com/ImageName:latest"
+                },
+            tags={}
         }
         result = client.start_execution(request)
         print(f"Execution started with ID: {result.execution_id}")
     """
 
-    def start_execution(self, request: StartExecutionRequest):
+    def start_execution(
+        self,
+        execution_name: str,
+        input_config: Dict[str, Any],
+        **kwargs,
+    ) -> dict:
+        # Convert underscores to hyphens in execution_name to meet regex validation
+        execution_name = execution_name.replace("_", "-")
+
+        parameters = {
+            "execution_name": execution_name,
+            "input_config": input_config,
+            **kwargs,
+        }
+
+        request = StartExecutionRequest(**parameters)
+        return self._start_execution_internal(request)
+
+    def _start_execution_internal(self, request: StartExecutionRequest):
+        self.__setup_execution_client()
         self.validate_execution_name(request.execution_name)
         self.validate_input_config(request.input_config)
         self.validate_client_token(request.get("client_token"))
@@ -405,12 +470,22 @@ class RemoteExecutionClient(ExecutionClient):
         local_input_file_path = request.input_config.get("notebook_config", {}).get(
             "input_path", ""
         )
+        is_git_project = RemoteExecutionUtils.is_git_project(
+            self.default_tooling_environment.get("provisionedResources")
+        )
         input_s3_location = RemoteExecutionUtils.pack_s3_path_for_input_file(
-            project_s3_path, local_input_file_path
+            project_s3_path,
+            local_input_file_path,
+            is_git_project,
+        )
+        local_full_input_file_path = RemoteExecutionUtils.pack_full_path_for_input_file(
+            local_input_file_path
         )
 
         output_s3_location = RemoteExecutionUtils.pack_s3_path_for_output_file(
-            project_s3_path, local_input_file_path
+            project_s3_path,
+            local_input_file_path,
+            is_git_project,
         )
 
         input = ExecutionUtils.unpack_path_file(input_s3_location)
@@ -423,7 +498,7 @@ class RemoteExecutionClient(ExecutionClient):
 
         if request.get("tags"):
             execution_tags.extend(
-                {"Key": key, "Value": value} for key, value in request.tags.items()
+                {"Key": key, "Value": value} for key, value in request.tags.to_dict().items()
             )
 
         if "compute" not in request:
@@ -434,9 +509,9 @@ class RemoteExecutionClient(ExecutionClient):
             ecr_uri = image_details["ecr_uri"]
         else:
             if self.datazone_stage == "prod":
-                image_name_default = "sagemaker-distribution-embargoed-prod"
+                image_name_default = "sagemaker-distribution-prod"
             else:
-                image_name_default = "sagemaker-distribution-embargoed-loadtest"
+                image_name_default = "sagemaker-distribution-loadtest"
             image_name: str = image_details.get("image_name", image_name_default)
             image_version: str = image_details.get("image_version", DEFAULT_IMAGE_VERSION)
             instance_type = request.compute.get("instance_type", DEFAULT_INSTANCE_TYPE)
@@ -460,7 +535,8 @@ class RemoteExecutionClient(ExecutionClient):
             if image_name != image_name_default:
                 # BYOI case, call describe-image-version api
                 response = self.sagemaker_client.describe_image_version(
-                    ImageName=image_name, Version=int(image_details.get("image_version"))
+                    ImageName=image_name,
+                    Version=int(image_details.get("image_version")),
                 )
                 ecr_uri = response.get("BaseImage")
             else:
@@ -475,11 +551,10 @@ class RemoteExecutionClient(ExecutionClient):
                 if account_id == "":
                     raise InternalServerError("Account ID not found in SSM parameter store")
                 # construct ECR uri of this format: <ACCOUNT>.dkr.ecr.<REGION>.amazonaws.<TLD>/sagemaker-distribution-prod:<TAG>
-                # example: 123456123456.dkr.ecr.us-west-2.amazonaws.com/sagemaker-distribution-prod:3.0.0-reinvent2024-cpu
+                # example: 123456123456.dkr.ecr.us-west-2.amazonaws.com/sagemaker-distribution-prod:3.0.0-cpu
                 # get the region from the default environment
                 region = self.default_tooling_environment["awsAccountRegion"]
-                image_version_suffix = "reinvent2024"
-                ecr_uri = f"{account_id}.dkr.ecr.{region}.amazonaws.com/{image_name}:{image_version}-{image_version_suffix}-{image_variant}"
+                ecr_uri = f"{account_id}.dkr.ecr.{region}.amazonaws.com/{image_name}:{image_version}-{image_variant}"
 
         output_formats = (
             request.get("output_config", {})
@@ -505,23 +580,64 @@ class RemoteExecutionClient(ExecutionClient):
                 kms_key_id = {"KmsKeyId": self.kms_key_identifier}
                 volume_kms_key_id = {"VolumeKmsKeyId": self.kms_key_identifier}
 
-            create_training_job_response = self.sagemaker_client.create_training_job(
-                TrainingJobName=f"{request.execution_name}-{uuid.uuid4()}",
-                AlgorithmSpecification={
+            # Define environment variables for SM Training job
+            environment_variables = {
+                "AWS_DEFAULT_REGION": self.default_tooling_environment["awsAccountRegion"],
+                "SM_EFS_MOUNT_GID": "100",
+                "SM_EFS_MOUNT_PATH": "/home/sagemaker-user",
+                "SM_EFS_MOUNT_UID": "1000",
+                "SM_ENV_NAME": "sagemaker-workflows-default-env",
+                "SM_EXECUTION_INPUT_PATH": "/opt/ml/input/data/sagemaker_workflows",
+                "SM_EXECUTION_SYSTEM_PATH": "/opt/ml/input/data/sagemaker_workflows_system",
+                "SM_INPUT_NOTEBOOK_NAME": input["file"],
+                "SM_JOB_DEF_VERSION": "1.0",
+                "SM_KERNEL_NAME": "python3",
+                "SM_OUTPUT_NOTEBOOK_NAME": output["file"],
+                "SM_SKIP_EFS_SIMULATION": "true",
+                "DataZoneDomainId": self.domain_identifier,
+                "DataZoneProjectId": self.project_identifier,
+                "DataZoneEndpoint": self.datazone_endpoint,
+                "DataZoneDomainRegion": self.datazone_domain_region,
+                "DataZoneStage": self.datazone_stage,
+                "DataZoneEnvironmentId": self.datazone_environment_id,
+                "InputNotebookPath": local_full_input_file_path,
+                "ProjectS3Path": self.project_s3_path,
+                "AWS_REGION": self.default_tooling_environment["awsAccountRegion"],
+                "SM_OUTPUT_FORMATS": (
+                    ",".join(output_formats_lowercase) if output_formats_lowercase else ""
+                ),
+                # Full path got init script in SM training container is `"{SM_EXECUTION_INPUT_PATH}/${SM_INIT_SCRIPT}"`
+                # The `sm_init_script.sh` is provided in SMD
+                "SM_INIT_SCRIPT": "../../../../../etc/sagemaker-ui/workflows/sm_init_script.sh",
+            }
+
+            training_job_params = {
+                "TrainingJobName": f"{request.execution_name}-{uuid.uuid4()}",
+                "AlgorithmSpecification": {
                     "TrainingImage": ecr_uri,
                     "TrainingInputMode": "File",
                     "EnableSageMakerMetricsTimeSeries": False,
                     "ContainerEntrypoint": ["amazon_sagemaker_scheduler"],
+                    "MetricDefinitions": [
+                        {
+                            "Name": "cells:complete",
+                            "Regex": r".*Executing:.*?\|.*?\|\s+(\d+)\/\d+\s+\[",
+                        },
+                        {
+                            "Name": "cells:total",
+                            "Regex": r".*Executing:.*?\|.*?\|\s+\d+\/(\d+)\s+\[",
+                        },
+                    ],
                 },
-                RoleArn=self.user_role_arn,
-                OutputDataConfig={"S3OutputPath": output["path"], **kms_key_id},
-                ResourceConfig={
+                "RoleArn": self.user_role_arn,
+                "OutputDataConfig": {"S3OutputPath": output["path"], **kms_key_id},
+                "ResourceConfig": {
                     "InstanceCount": 1,
                     "InstanceType": request.compute.get("instance_type", ""),
                     "VolumeSizeInGB": request.compute.get("volume_size_in_gb", 30),
                     **volume_kms_key_id,
                 },
-                InputDataConfig=[
+                "InputDataConfig": [
                     {
                         "ChannelName": "sagemaker_workflows",
                         "DataSource": {
@@ -535,49 +651,33 @@ class RemoteExecutionClient(ExecutionClient):
                         "CompressionType": "None",
                     }
                 ],
-                HyperParameters=request.input_config.get("notebook_config", {}).get(
+                "HyperParameters": request.input_config.get("notebook_config", {}).get(
                     "input_parameters", {}
                 ),
-                StoppingCondition={
+                "StoppingCondition": {
                     "MaxRuntimeInSeconds": request.get("termination_condition", {}).get(
                         "max_runtime_in_seconds", 86400
                     )
                 },
-                EnableManagedSpotTraining=False,
-                EnableNetworkIsolation=False,
-                EnableInterContainerTrafficEncryption=True,
-                Environment={
-                    "AWS_DEFAULT_REGION": self.default_tooling_environment["awsAccountRegion"],
-                    "SM_EFS_MOUNT_GID": "100",
-                    "SM_EFS_MOUNT_PATH": "/home/sagemaker-user",
-                    "SM_EFS_MOUNT_UID": "1000",
-                    "SM_ENV_NAME": "sagemaker-workflows-default-env",
-                    "SM_EXECUTION_INPUT_PATH": "/opt/ml/input/data/sagemaker_workflows",
-                    "SM_EXECUTION_SYSTEM_PATH": "/opt/ml/input/data/sagemaker_workflows_system",
-                    "SM_INPUT_NOTEBOOK_NAME": input["file"],
-                    "SM_JOB_DEF_VERSION": "1.0",
-                    "SM_KERNEL_NAME": "python3",
-                    "SM_OUTPUT_NOTEBOOK_NAME": output["file"],
-                    "SM_SKIP_EFS_SIMULATION": "true",
-                    "DataZoneDomainId": self.domain_identifier,
-                    "DataZoneProjectId": self.project_identifier,
-                    "DataZoneEndpoint": self.datazone_endpoint,
-                    "DataZoneDomainRegion": self.datazone_domain_region,
-                    "DataZoneStage": self.datazone_stage,
-                    "DataZoneEnvironmentId": self.datazone_environment_id,
-                    "ProjectS3Path": self.project_s3_path,
-                    "AWS_REGION": self.default_tooling_environment["awsAccountRegion"],
-                    "SM_OUTPUT_FORMATS": (
-                        ",".join(output_formats_lowercase) if output_formats_lowercase else ""
-                    ),
-                    # Full path got init script in SM training container is `"{SM_EXECUTION_INPUT_PATH}/${SM_INIT_SCRIPT}"`
-                    # The `sm_init_script.sh` is provided in SMD
-                    "SM_INIT_SCRIPT": "../../../../../etc/sagemaker-ui/workflows/sm_init_script.sh",
-                },
-                RetryStrategy={"MaximumRetryAttempts": 1},
-                Tags=execution_tags,
-                VpcConfig={"SecurityGroupIds": [self.security_group], "Subnets": self.subnets},
+                "EnableManagedSpotTraining": False,
+                "EnableNetworkIsolation": False,
+                "EnableInterContainerTrafficEncryption": True,
+                "Environment": environment_variables,
+                "RetryStrategy": {"MaximumRetryAttempts": 1},
+                "Tags": execution_tags,
+            }
+
+            # Add VpcConfig only if both security_group and subnets are available
+            if self.security_group is not None and self.subnets is not None:
+                training_job_params["VpcConfig"] = {
+                    "SecurityGroupIds": [self.security_group],
+                    "Subnets": self.subnets,
+                }
+
+            create_training_job_response = self.sagemaker_client.create_training_job(
+                **training_job_params
             )
+
             split_arn = create_training_job_response.get("TrainingJobArn").split(":training-job/")
             if len(split_arn) != 2:
                 raise RuntimeError("Remote executionId not available")
@@ -603,9 +703,7 @@ class RemoteExecutionClient(ExecutionClient):
     its current state.
 
     Args:
-        request (StopExecutionRequest): An object containing the necessary information to identify
-        and stop the execution. This typically includes:
-        - execution_id: The unique identifier of the execution to stop
+        execution_id (str): The unique identifier of the execution to stop
 
     Returns:
         None: This method doesn't return any value. The absence of an exception indicates
@@ -628,7 +726,13 @@ class RemoteExecutionClient(ExecutionClient):
             print("Execution not found")
     """
 
-    def stop_execution(self, request: StopExecutionRequest) -> None:
+    def stop_execution(self, execution_id: str) -> None:
+        parameters = {"execution_id": execution_id}
+        request = StopExecutionRequest(**parameters)
+        return self._stop_execution_internal(request)
+
+    def _stop_execution_internal(self, request: StopExecutionRequest) -> None:
+        self.__setup_execution_client()
         try:
             self.sagemaker_client.stop_training_job(
                 TrainingJobName=request.execution_id,
@@ -668,7 +772,8 @@ class RemoteExecutionClient(ExecutionClient):
 
     def __set_sagemaker_environment(self):
         self.sagemaker_environment = self.project_api.get_project_sagemaker_environment(
-            domain_identifier=self.domain_identifier, project_identifier=self.project_identifier
+            domain_identifier=self.domain_identifier,
+            project_identifier=self.project_identifier,
         )
         return
 
@@ -747,6 +852,47 @@ class RemoteExecutionClient(ExecutionClient):
             raise RuntimeError("Default environment provisioned resources not found")
         return
 
+    def __setup_execution_client(self):
+        if self.domain_identifier is None:
+            self.domain_identifier = self.config.domain_identifier
+        if self.project_identifier is None:
+            self.project_identifier = self.config.project_identifier
+        if not self.config.domain_identifier or not self.config.project_identifier:
+            raise InternalServerError("Domain identifier and project identifier are required")
+
+        if self.datazone_stage is None:
+            self.datazone_stage = self.config.datazone_stage
+        if self.datazone_endpoint is None:
+            self.datazone_endpoint = self.config.datazone_endpoint
+        if self.datazone_domain_region is None:
+            self.datazone_domain_region = self.config.datazone_domain_region
+        if self.project_s3_path is None:
+            self.project_s3_path = self.config.project_s3_path
+        if self.datazone_environment_id is None:
+            if not self.config.datazone_environment_id:
+                raise InternalServerError("DataZone environment id is required")
+            self.datazone_environment_id = self.config.datazone_environment_id
+
+        if self.default_tooling_environment is None:
+            self.__set_default_tooling_environment()
+        if self.sagemaker_environment is None:
+            self.__set_sagemaker_environment()
+        if self.sagemaker_client is None:
+            self.__set_sagemaker_client()
+        if self.ec2_client is None:
+            self.__set_ec2client()
+        if self.ssm_client is None:
+            self.__set_ssmclient()
+        if None in [
+            self.security_group,
+            self.subnets,
+            self.user_role_arn,
+            self.kms_key_identifier,
+        ]:
+            self.__set_stack()
+
+        return
+
     @staticmethod
     def validate_execution_name(execution_name: str):
         if not re.match(r"^[a-zA-Z0-9]([-a-zA-Z0-9]){0,25}$", execution_name):
@@ -789,10 +935,6 @@ class RemoteExecutionClient(ExecutionClient):
             raise ValidationError("Client (idempotency) token not supported")
 
     def __validate_stack(self):
-        if self.security_group is None:
-            raise RuntimeError("Default stack security_group not found")
-        if self.subnets is None:
-            raise RuntimeError("Default stack subnets not found")
         if self.user_role_arn is None:
             raise RuntimeError("Default stack use_role_arn not found")
         return
@@ -812,9 +954,12 @@ class RemoteExecutionClient(ExecutionClient):
             if parsed_version.major == 3:
                 return True
 
-            # Valid for 2.2.x versions and 2.2 or 2
+            # Since 2025 June, switching to public smd images, valid versions are 2.6.x, 2.7.x, 2.8.x, 2.9.x
             if parsed_version.major == 2 and (
-                parsed_version.minor == 2 or parsed_version.base_version == "2"
+                parsed_version.minor == 6
+                or parsed_version.minor == 7
+                or parsed_version.minor == 8
+                or parsed_version.base_version == "2"
             ):
                 return True
             return False
